/*
  Layer Bookmark
    あかつきみさき

  このスクリプトについて
    指定したレイヤーをリストに登録し,リスト選択でレイヤーを選択状態にするスクリプト.

  使用方法
    1.ファイル→スクリプト→スクリプトファイルの実行から実行.
    2.ScriptUIフォルダに入れてから,ウィンドウより実行.

  動作保障環境
    Adobe After Effects 22.0以上

  バージョン情報
    2021/12/21 Ver 1.0.0 Release
*/

(function() {
    if (app.project.layerByID == undefined) {
          alert("This script requires After Effects 22.0 or greater." + "\nCurrent version: " + parseFloat(app.version), "Layer Bookmark");
        return 0;
    }

    function buildUI(thisObj) {
        var layerbmWin;
        if (thisObj instanceof Panel) {
            layerbmWin = thisObj;
        } else {
            layerbmWin = new Window("palette", "Layer Bookmark", undefined, {
                resizeable: true
            });
        }

        layerbmWin.orientation = "column";
        layerbmWin.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.TOP];
        layerbmWin.margins = [0, 0, 0, 0];
        layerbmWin.spacing = 1;
        layerbmWin.size = [400, 500];

        var btnGrp = layerbmWin.add("group");
        btnGrp.margins = [0, 0, 0, 0];
        btnGrp.spacing = 0;
        btnGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.TOP];
        btnGrp.orientation = "row";

        var addBtn = btnGrp.add("button", undefined, "Add");
        addBtn.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        addBtn.helpTip = "";

        var clearBtn = btnGrp.add("button", undefined, "Clear");
        clearBtn.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        clearBtn.helpTip = "";

        var helpBtn = btnGrp.add("button", undefined, "?");
        helpBtn.alignment = [ScriptUI.Alignment.RIGHT, ScriptUI.Alignment.FILL];
        helpBtn.minimumSize.width = helpBtn.maximumSize.width = 25;
        helpBtn.helpTip = "";

        var listGrp = layerbmWin.add("group");
        listGrp.margins = [0, 0, 0, 0];
        listGrp.spacing = 0;
        listGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        listGrp.orientation = "column";

        var proLb = listGrp.add("listbox", undefined, undefined, {
            numberOfColumns: 4,
            showHeaders: true,
            columnTitles: ["", "Layer", "Comp"]
        });

        proLb.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];

        addBtn.onClick = function() {
            var itemList = [];

            var actComp = app.project.activeItem;
            if (!(actComp instanceof CompItem)) {
                return 0;
            }

            var selLayers = actComp.selectedLayers;
            for (var j = 0; j < selLayers.length; j++) {
                var added = false;
                for (var i = proLb.items.length - 1; i >= 0; i--) {
                    if (proLb.items[i].layerID == selLayers[j].id) {
                        added = true;
                        continue;
                    }
                }

                if (added) {
                    continue;
                }

                itemList[itemList.length] = proLb.add("item", "");

                itemList[itemList.length - 1].checked = false;
                itemList[itemList.length - 1].subItems[0].text = selLayers[j].name;
                itemList[itemList.length - 1].subItems[1].text = actComp.name;

                itemList[itemList.length - 1].comp = actComp;
                itemList[itemList.length - 1].layerID = selLayers[j].id;

            }
        }

        clearBtn.onClick = function() {
            for (var i = proLb.items.length - 1; i >= 0; i--) {
                if (proLb.items[i].checked) {
                    continue;
                }

                proLb.remove(proLb.items[i]);
            }
        }

        proLb.onChange = function() {
            if (this.selection == null) {
                return 0;
            }

            if (ScriptUI.environment.keyboardState.ctrlKey) {
                this.selection.checked = !this.selection.checked;

                this.selection = null;
                return 0;
            }

            if (ScriptUI.environment.keyboardState.altKey) {
                proLb.remove(this.selection);
                this.selection = null;
                return 0;
            }

            this.selection.comp.openInViewer();

            for (var i = this.selection.comp.selectedLayers.length - 1; i >= 0; i--) {
                this.selection.comp.selectedLayers[i].selected = false;
            }

            var targetLayer = app.project.layerByID(this.selection.layerID);
            if (targetLayer != null) {
                targetLayer.selected = true;
            }

            this.selection = null;
        }

        helpBtn.onClick = function() {
            alert("Layer Bookmark 0.0.1\nあかつきみさき\n\n選択したレイヤーをリストに登録し,リストクリックで該当レイヤーを選択するスクリプト.\n\nクリック: 該当レイヤーの選択\nCtrl+クリック: 該当項目の保護(チェックマーク)\nAlt+クリック: 該当項目の削除\nAdd: 選択レイヤーをリストに反映\nClear: 保護されていないリスト項目をクリア", "Layer Bookmark");
        }

        layerbmWin.layout.layout();
        layerbmWin.onResizing = layerbmWin.onResize = function() {
            this.layout.resize();
        }

        if (!(thisObj instanceof Panel)) {
            layerbmWin.center();
            layerbmWin.show();
        }

        return 0;
    }

    buildUI(this);

    return 0;
}).call(this);